#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#define RQ_Soil_Moisture 1
#define RQ_Temp_Sensor 2
#define RQ_Light_Sensor 3
#define RQ_Relay 4
#define RQ_moter 5
#define RQ_PC 6

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint16_t Soil_Moisture;
    static int16_t temperature;
    static uint16_t lumen;
    static uint8_t pin_value;
    uint8_t relay_state;
    uint8_t relay_no;
    uint8_t moter_state;
    uint8_t pin;

    switch(rq -> bRequest){
      case RQ_Soil_Moisture:
        Soil_Moisture = read_adc(PC4);
        usbMsgPtr = &Soil_Moisture;
        return 2;
      case RQ_Temp_Sensor:
        temperature = read_adc(PC3);
        usbMsgPtr = &temperature;      
        return 2;
      case RQ_Light_Sensor:
        lumen = read_adc(PC2);
        usbMsgPtr = &lumen;
        return 2; 
      case RQ_Relay:
        relay_state = rq->wValue.bytes[0];
        set_relay(relay_state);
        return 1;
      case RQ_moter:
        moter_state = rq->wValue.bytes[0];
        set_moter(moter_state);
        return 1;
      case RQ_PC:
        pin = rq->wIndex.bytes[0]; 
        pin_value = PORTC & (1<<pin);
        usbMsgPtr = &pin_value;
        return 1;
    }

    /* default for not implemented requests: return no data back to host */
    return 0;
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peri();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
